/**
 * Copyright (C) 2008 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <net/if.h>
#include <sys/ioctl.h>
#include <sys/statfs.h>
#include <mntent.h>

#include "main.h"

#define MNT "/proc/mounts"

enum /* persistence */
{
    LPP_ERROR = 0,
    LPP_HOME,
    LPP_ALL,
    LPP_NONE,
};

const gchar *persist_char[4] = {
    N_("Unable to find the status."),
    N_("User dir persistence only"),
    N_("Full persistence"),
    N_("Lock mode (RAM system)")
};

static void
get_ifaces (GHashTable *hash)
{
    char buf[1024];
    struct ifconf ifc;
    struct ifreq *ifr;
    int fd;
    int i, ni;

    fd = socket(AF_INET,SOCK_DGRAM, 0);
    ifc.ifc_len = sizeof(buf);
    ifc.ifc_buf = buf;
    ioctl (fd, SIOCGIFCONF, &ifc);
    ifr = ifc.ifc_req;
    ni = ifc.ifc_len / sizeof (struct ifreq);

    for (i=0;i<ni;i++)
    {
        struct ifreq *item = &ifr[i];
        if (ioctl(fd, SIOCGIFADDR, item) == 0)
        {
            /* forget about lo */
            if (!strcmp (item->ifr_name, "lo"))
                continue;

            gchar ip[16];
            g_sprintf (ip, "%d.%d.%d.%d",
                (unsigned char) item->ifr_addr.sa_data[2],
                (unsigned char) item->ifr_addr.sa_data[3],
                (unsigned char) item->ifr_addr.sa_data[4],
                (unsigned char) item->ifr_addr.sa_data[5]);
            g_hash_table_insert (hash, g_strdup (item->ifr_name), g_strdup (ip));
        }
    }
}

static void
str_from_hash (gpointer key, gpointer value, gchar ** data)
{
    gchar *net_devs = NULL;
    gchar *tmp = *data;
    net_devs = g_strconcat (tmp ? tmp : "",
                            (gchar *) key, " => ",
                            (gchar *) value,"\n",
                            NULL);
    g_free (tmp);
    *data = net_devs;
}

static void
get_infos (guint *persistence, gchar **device, gchar **freestr)
{
    FILE *fp;
    struct mntent *fs;
    struct statfs state;
    glong freespace = -1;
    gboolean have_home = FALSE;
    gboolean have_cow = FALSE;
    gboolean have_rofsbis = FALSE;
    gchar *roottmp = NULL;
    gchar *persisttmp = NULL;

    fp = setmntent (MNT, "r");
    if (G_UNLIKELY (!fp))
        return;

    while ((fs = getmntent (fp)) != NULL)
    {
        if (strncmp (fs->mnt_dir, "/cdrom", 6) == 0)
            roottmp = g_strndup (fs->mnt_fsname, 8); /* /dev/[hs]d? */
        if (strncmp (fs->mnt_dir, "/home", 5) == 0)
        {
            have_home = TRUE;
            persisttmp = g_strdup (fs->mnt_dir);
        }
        if (strncmp (fs->mnt_dir, "/cow", 4) == 0)
        {
            have_cow = TRUE;
            persisttmp = g_strdup ("/");
        }
        if (strncmp (fs->mnt_dir, "/rofsbis", 8) == 0)
        {
            have_rofsbis = TRUE;
            persisttmp = g_strdup ("/");
        }
    }
    endmntent (fp);

    *persistence = LPP_ERROR;
    if (have_home)
        *persistence = LPP_HOME;
    else if (have_cow)
    {
        if (have_rofsbis)
            *persistence = LPP_NONE; /* lock mode */
        else
            *persistence = LPP_ALL;
    }

    *device = roottmp;

    if (persisttmp)
    {
        if (!statfs (persisttmp, &state))
            freespace = state.f_bavail / 1024 * state.f_bsize;
    }
    if (freespace >= 1000000)
        *freestr = g_strdup_printf ("%ldG", freespace / 1000000);
    else if (freespace >= 1000)
        *freestr = g_strdup_printf ("%ldM", freespace / 1000);
    else if (freespace < 0)
        *freestr = g_strdup_printf (_("Unable to determine"));
    else
        *freestr = g_strdup_printf (_("No space left"));

    g_free (persisttmp);
}

GtkWidget *
get_info_widget ()
{
    GtkWidget *table, *vbox, *label, *hs;
    GHashTable *hash;
    gchar *device = NULL;
    gchar *freespace = NULL;
    guint persistence = LPP_ERROR;
    gchar *net_devs = NULL;

    hash = g_hash_table_new_full (g_str_hash, g_str_equal,
                                  g_free, g_free);
    get_ifaces (hash);
    get_infos (&persistence, &device, &freespace);

    vbox = gtk_vbox_new (FALSE, 0);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\" size=\"large\">System Configuration</span>"));
    gtk_misc_set_alignment (GTK_MISC (label), 0.5, 0);
    gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 8);

    hs = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (vbox), hs, FALSE, FALSE, 8);

    table = gtk_table_new (4, 2, FALSE);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\">Root device: </span>"));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 0, 1,
                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 1, 0);

    label = gtk_label_new (device);
    gtk_table_attach (GTK_TABLE (table), label, 1, 2, 0, 1,
                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\">Free disk space: </span>"));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 1, 0);

    label = gtk_label_new (freespace);
    gtk_table_attach (GTK_TABLE (table), label, 1, 2, 1, 2,
                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\">Persistence: </span>"));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 1, 0);

    label = gtk_label_new (persist_char[persistence]);
    gtk_table_attach (GTK_TABLE (table), label, 1, 2, 2, 3,
                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("<span weight=\"bold\">Network Interfaces: </span>"));
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 3, 4,
                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 1, 0);
    g_hash_table_foreach (hash, (GHFunc) str_from_hash, &net_devs);
    label = gtk_label_new (net_devs);
    g_hash_table_destroy (hash);
    g_free (net_devs);

    gtk_table_attach (GTK_TABLE (table), label, 1, 2, 3, 4,
                      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0);

    gtk_box_pack_start (GTK_BOX (vbox), table, TRUE, TRUE, 2);
    gtk_widget_show_all (vbox);

    return vbox;
}

static void
on_url_clicked (GtkWidget *btn, gpointer data)
{
    if (app_is_usable ("firefox"))
        run_user_cmd ("firefox http://linutop.com/wiki", TRUE);
}

GtkWidget *
get_about_widget ()
{
    GtkWidget *box, *hs;
    GtkWidget *label, *btn;

    box = gtk_vbox_new (FALSE, 2);

    hs = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (box), hs, FALSE, FALSE, 8);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        "<span size=\"large\" weight=\"bold\">"
        "Linutop Setup Menu V. " PACKAGE_VERSION
        "</span>");
    gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 2);

    hs = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (box), hs, FALSE, FALSE, 8);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("Designed for quick and simple setup "
          "of your Linutop computer."));
    gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 2);

    btn = gtk_link_button_new_with_label ("http://linutop.com/wiki",
        _("More on http://linutop.com/wiki"));
    gtk_button_set_relief (GTK_BUTTON (btn), GTK_RELIEF_NONE);
    g_signal_connect (G_OBJECT (btn), "clicked",
        G_CALLBACK (on_url_clicked), NULL);
    gtk_box_pack_start (GTK_BOX (box), btn, FALSE, FALSE, 2);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label),
        _("Thank you for using Linutop!\n"));
    gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 2);

    gtk_widget_show_all (box);

    return box;
}
