/* pwd.c
 * Copyright (C) 2007-2008 Gauvain Pocentek <gpocentek@linutop.com>
 * This file is part of the linutop config editor
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, version 2 of the license.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define _XOPEN_SOURCE
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "main.h"

enum {
    OK = 0,
    CANCEL
};

typedef struct _pw {
    GtkWidget *entry1;
    GtkWidget *entry2;
    GtkWidget *okbtn;
    gchar *pwd;
} PW;

gchar *
get_hash (const gchar *clear)
{
    const char s[] =
        "azertyuiopmlkjhgfdsqwxcvbnAZERTYUIOPMLKJHGFDSQWXCVBN./012 3456789";
    gchar salt[] = "$1$XXXXXXXX$";

    int i;
    for (i = 3; i < 11; i++)
        salt[i] = s[rand () % (sizeof (s))];
    return g_strdup (crypt (clear, salt));
}

static void
pwd_ok (GtkWidget *entry, gpointer data)
{
    PW *pwds = (PW *) data;
    gchar *t1 = NULL, *t2 = NULL;
    gchar *tmp = NULL;

    t1 = g_strdup (gtk_entry_get_text (GTK_ENTRY (pwds->entry1)));
    t2 = g_strdup (gtk_entry_get_text (GTK_ENTRY (pwds->entry2)));
    if (strlen (t1) && strlen (t2) && (strcmp (t1, t2) == 0)) {
        gtk_widget_set_sensitive (pwds->okbtn, TRUE);
        tmp = get_hash (t1);
        g_free (pwds->pwd);
        pwds->pwd = tmp;
    } else
        gtk_widget_set_sensitive (pwds->okbtn, FALSE);
    g_free (t1);
    g_free (t2);
}

/* returns the md5 hash */
gchar *
get_lock_passwd ()
{
    gchar *ret = NULL;
    GtkWidget *w, *label, *table;
    GtkWidget *vbox, *hbox, *img;
    GtkWidget *frame;
    GtkWidget *cancelbtn;

    PW *pwds = g_new0 (PW, 1);

    w = gtk_dialog_new ();
    gtk_window_set_title (GTK_WINDOW (w), _("Administrator password"));
    gtk_container_set_border_width (GTK_CONTAINER (GTK_DIALOG (w)->vbox), 6);

    vbox = gtk_vbox_new (FALSE, 2);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label), _(
        "Two accounts are enabled when using the locked mode:\n"
        " - the 'linutop' account has system administration rights\n"
        " - the 'user' account has no privileges at all, and can only\n"
        "   use applications.\n"
        "Switching from one account to the other can be done using\n"
        "the logout dialog.\n\n"
        "To secure the system, a password is needed to login with the\n"
        "administrator account."));
    gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 2);
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);

    hbox = gtk_hbox_new (FALSE, 0);
    gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 2);

    img = gtk_image_new_from_stock (GTK_STOCK_DIALOG_WARNING,
                                    GTK_ICON_SIZE_DIALOG);
    gtk_misc_set_alignment (GTK_MISC (img), 0.5, 0);
    gtk_box_pack_start (GTK_BOX (hbox), img, FALSE, FALSE, 2);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label), _(
        "<span weight=\"bold\">"
        "Don't forget this password, and make sure to\n"
        "create a backup of the system (Advanced tab)!</span>"));
    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 2);

    table = gtk_table_new (3, 2, FALSE);
    gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 2);

    label = gtk_label_new (NULL);
    gtk_label_set_markup (GTK_LABEL (label), _(
        "<span style=\"italic\">Select a password for the 'linutop' "
        "(administrator) account:</span>"));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 2, 0, 1,
             GTK_FILL, GTK_FILL, 2, 2);

    pwds->entry1 = gtk_entry_new_with_max_length (16);
    gtk_entry_set_visibility (GTK_ENTRY (pwds->entry1), FALSE);
    gtk_table_attach (GTK_TABLE (table), pwds->entry1, 1, 2, 1, 2,
             GTK_EXPAND | GTK_FILL, GTK_FILL, 0, 0);
    pwds->entry2 = gtk_entry_new_with_max_length (16);
    gtk_entry_set_visibility (GTK_ENTRY (pwds->entry2), FALSE);
    gtk_table_attach (GTK_TABLE (table), pwds->entry2, 1, 2, 2, 3,
             GTK_FILL, GTK_FILL, 0, 0);
    label = gtk_label_new (_("Password: "));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2,
             GTK_FILL, GTK_SHRINK, 0, 0);
    label = gtk_label_new (_("Retype password: "));
    gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
    gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3,
             GTK_FILL, GTK_SHRINK, 0, 0);

    frame = xfce_framebox_new (_("Linutop accounts setup"), TRUE);
    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_NONE);
    xfce_framebox_add (XFCE_FRAMEBOX (frame), vbox);
    gtk_container_add (GTK_CONTAINER (GTK_DIALOG (w)->vbox), frame);

    cancelbtn = gtk_button_new_from_stock (GTK_STOCK_CANCEL);
    pwds->okbtn = gtk_button_new_from_stock (GTK_STOCK_OK);
    gtk_widget_set_sensitive (pwds->okbtn, FALSE);
    gtk_dialog_add_action_widget (GTK_DIALOG (w), cancelbtn, CANCEL);
    gtk_dialog_add_action_widget (GTK_DIALOG (w), pwds->okbtn, OK);

    g_signal_connect (G_OBJECT (pwds->entry1), "changed",
             G_CALLBACK (pwd_ok), pwds);
    g_signal_connect (G_OBJECT (pwds->entry2), "changed",
             G_CALLBACK (pwd_ok), pwds);

    gtk_widget_show_all (w);
    int response = gtk_dialog_run (GTK_DIALOG (w));
    if (response == OK && pwds->pwd) {
        ret = g_strdup (pwds->pwd);
    }
    if (pwds->pwd)
    {
        memset (pwds->pwd, 0, sizeof (gchar) * strlen (pwds->pwd));
        g_free (pwds->pwd);
    }
    gtk_widget_destroy (w);
    g_free (pwds);

    return ret;
}
