/**
 * Copyright (C) 2009 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <curl/curl.h>

#include "main.h"

#define BASE_URL "https://webupdate.linutop.eu/"
#define ACCOUNT_URL BASE_URL "myaccount.php"
#define CFG_URL BASE_URL "cfg.php"
#define PREPUPDATE_URL BASE_URL "prepupdate.php"

static size_t
write_callback (void *ptr, size_t size, size_t nmemb, void **data)
{
    if (! *data)
        *data = g_strdup ((gchar *) ptr);
    else
    {
        gchar *tmp = g_strconcat (*data, (gchar *) ptr, NULL);
        g_free (*data);
        *data = tmp;
    }

    return strlen (*data);
}

void
free_status (status *s)
{
    if (s->error)
        g_free (s->error);
    else
        g_hash_table_destroy (s->images);

    g_free (s);
}

gboolean
get_page (const gchar *url, gchar **output, const gchar *firstkey, ...)
{
    va_list args;
    const gchar *key;
    const gchar *value;
    CURL *curl;
    gchar *data = NULL;
    gchar *tmp = NULL;
    gchar *buffer = NULL;

    va_start (args, firstkey);
    key = firstkey;
    value = va_arg (args, const char *);
    while (key != NULL) {
        if (! data) {
            data = g_strconcat (key, "=", value, NULL);
        } else {
            tmp = g_strconcat (data, "&", key, "=", value, NULL);
            g_free (data);
            data = tmp;
        }

        key = va_arg (args, const char *);
        value = va_arg (args, const char *);
    }
    va_end (args);

    curl = curl_easy_init ();
    if (! curl)
        return FALSE;

    if (data)
        curl_easy_setopt (curl, CURLOPT_POSTFIELDS, data);
    curl_easy_setopt (curl, CURLOPT_URL, url);
    curl_easy_setopt (curl, CURLOPT_WRITEFUNCTION, (curl_write_callback) write_callback);
    curl_easy_setopt (curl, CURLOPT_WRITEDATA, (void *)&buffer);
    /* We don't have a CA cert so don't check it */
    curl_easy_setopt(curl, CURLOPT_SSL_VERIFYPEER, 0L);

    curl_easy_perform (curl);

    g_free (*output);
    *output = buffer;

    curl_easy_cleanup (curl);
    g_free (data);

    return TRUE;
}

status *
get_informations (const gchar *login, const gchar *passwd, const gchar *os)
{
    gchar *buf = NULL;
    status *s;

    if (!login && !passwd)
    {
        if (!get_page (ACCOUNT_URL, &buf, "os", os, NULL))
        {
            g_free (buf);
            return NULL;
        }
    } else {
        if (!get_page (ACCOUNT_URL, &buf,
                       "login", login, "passwd", passwd, "os", os, NULL))
        {
            g_free (buf);
            return NULL;
        }
    }

    if (!buf)
        return NULL;

    s = (status *) malloc (sizeof (status));
    if (!s)
        return NULL;

    parse_status_page (buf, s);
    g_free (buf);

    return s;
}

token *
get_token (const gchar *login, const gchar *passwd, const gchar *imageid)
{
    gchar *buf = NULL;
    token *t;

    if (!login && !passwd)
    {
        if (!get_page (PREPUPDATE_URL, &buf, "imageid", imageid, NULL))
        {
            g_free (buf);
            return NULL;
        }
    } else {
        if (!get_page (PREPUPDATE_URL, &buf, "login", login,
                       "passwd", passwd, "imageid", imageid, NULL))
        {
            g_free (buf);
            return NULL;
        }
    }

    if (!buf)
        return NULL;

    t = (token *) malloc (sizeof (t));
    if (!t)
        return NULL;

    parse_prepupdate_page (buf, t);
    g_free (buf);

    return t;
}

gboolean
get_cfg ()
{
    gchar *buf = NULL;
    if (!get_page (CFG_URL, &buf, NULL))
        return FALSE;

    if (!buf)
        return FALSE;

    return parse_cfg_page (buf);
}
