/**
 * Copyright (C) 2009 Gauvain Pocentek <gpocentek@linutop.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <glib.h>
#include <glib/gstdio.h> /* g_access */

#include "main.h"

#define LSB_RELEASE "/etc/lsb-release"

/* strings */
gint
str_cmp_func (GtkTreeModel *model,
              GtkTreeIter *a,
              GtkTreeIter *b,
              gpointer data)
{
    gint ret;
    gchar *str1 = NULL;
    gchar *str2 = NULL;

    gtk_tree_model_get(model, a, 1, &str1, -1);
    gtk_tree_model_get(model, b, 1, &str2, -1);

    ret = strcmp (str1, str2);

    g_free (str1);
    g_free (str2);

    return ret;
}

const gchar *
get_os ()
{
    XfceRc *rc;
    rc = xfce_rc_simple_open (LSB_RELEASE, TRUE);
    if (! rc)
        return "";

    const gchar *ret = g_strdup (
        xfce_rc_read_entry (rc, "LINUTOP_CODENAME", ""));
    xfce_rc_close (rc);

    return ret;
}

/* web pages parsing */
void
parse_status_page (gchar *buffer, status *s)
{
    int i, start = 0;
    gchar **lines = g_strsplit (buffer, "\n", 0);
    if (g_strv_length (lines) == 0)
    {
        g_strfreev (lines);
        return;
    }

    gchar **tmp = g_strsplit (lines[0], ":", 0);
    if (g_strv_length (tmp) != 2)
        goto cleanup;

    s->images = g_hash_table_new_full (g_str_hash, g_str_equal,
                                       NULL, g_free);
    s->error = NULL;
    s->count = -1;

    if (!strcmp ("error", tmp[0]))
    {
        s->error = g_strdup (tmp[1]);
        goto cleanup;
    } else if (! strcmp ("count", tmp[0])) {
        sscanf (tmp[1], "%d", &s->count);
        start = 1; /* to not reparse the first line */
    }
    if (g_strv_length (lines) < 2 + start)
        goto cleanup;

    for (i=start;i<g_strv_length (lines)-1;i++)
    {
        g_strfreev (tmp);
        tmp = g_strsplit (lines[i], ":", 0);
        if (strcmp (tmp[0], "image"))
            continue;

        gchar **tmp2 = g_strsplit (tmp[1], " ", 2);
        if (g_strv_length (tmp2) < 2)
        {
            g_strfreev (tmp2);
            continue;
        }

        g_hash_table_insert (s->images, g_strdup (tmp2[0]), g_strdup (tmp2[1]));
        g_strfreev (tmp2);
    }

cleanup:
    g_strfreev (lines);
    g_strfreev (tmp);
    return;
}

void
parse_prepupdate_page (gchar *buffer, token *t)
{
    gchar **lines = g_strsplit (buffer, "\n", 0);
    if (g_strv_length (lines) == 0)
    {
        g_strfreev (lines);
        return;
    }

    gchar **tmp = g_strsplit (lines[0], ":", 0);
    if (g_strv_length (tmp) != 2)
        goto cleanup;

    t->error = NULL;
    t->token = NULL;

    if (!strcmp ("error", tmp[0]))
        t->error = g_strdup (tmp[1]);
    else if (!strcmp ("token", tmp[0]))
        t->token = g_strdup (tmp[1]);

cleanup:
    g_strfreev (lines);
    g_strfreev (tmp);
    return;
}

gboolean
parse_cfg_page (gchar *buffer)
{
    int i;
    gchar **tmp = NULL;
    gchar **lines = NULL;

    cfg = g_hash_table_new_full (g_str_hash, g_str_equal,
                                 g_free, g_free);

    lines = g_strsplit (buffer, "\n", 0);
    if (g_strv_length (lines) == 0)
    {
        g_strfreev (lines);
        return FALSE;
    }

    for (i=0;i<g_strv_length (lines)-1;i++)
    {
        tmp = g_strsplit (lines[i], ":", 2);
        if (g_strv_length (tmp) != 2)
            continue;
        g_hash_table_insert (cfg, g_strdup (tmp[0]), g_strdup (tmp[1]));
        g_strfreev (tmp);
    }

    g_strfreev (lines);
    return TRUE;
}

/* config utils */
gboolean
config_is (const gchar *key, const gchar *value)
{
    const gchar *tmp = g_hash_table_lookup (cfg, key);
    if (tmp && !strncmp (tmp, value, strlen (value)))
        return TRUE;
    return FALSE;
}

gboolean
save_config (const gchar *firstkey, ...)
{
    if (! g_access (CONFFILE, W_OK))
        unlink (CONFFILE);

    if (!firstkey)
        return TRUE;

    va_list args;
    const gchar *key;
    const gchar *value;

    XfceRc *rc;

    rc = xfce_rc_simple_open (CONFFILE, FALSE);
    if (!rc)
        return FALSE;

    va_start (args, firstkey);
    key = firstkey;
    value = va_arg (args, const gchar *);
    while (key != NULL) {
        xfce_rc_write_entry (rc, key, value);

        key = va_arg (args, const gchar *);
        value = va_arg (args, const gchar *);
    }
    va_end (args);
    xfce_rc_close (rc);

    return TRUE;
}

/* prep-update helpers */
gboolean
get_prep_is_done ()
{
    int ret = system ("sh " LIBEXECDIR "/prep-update.sh get");
    /* The script returns 1 if the system is ready to be updated */
    if (ret)
        return TRUE;
    return FALSE;
}

gboolean
set_prep (gboolean prep)
{
    gchar *cmd = NULL;
    int ret;
    cmd = g_strdup_printf ("sh %s/prep-update.sh %s", LIBEXECDIR, prep ? "on" : "off");
    ret = system (cmd);
    g_free (cmd);
    return ret ? FALSE : TRUE;
}

/* GTK tools */
void
warn_service_down ()
{
    GtkWidget *warn = gtk_message_dialog_new (
        NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
        "The service is not available. Please try again later.");
    gtk_window_set_title (GTK_WINDOW (warn), "Service unavailable");
    gtk_dialog_run (GTK_DIALOG (warn));
}

void
warn_setup_error ()
{
    GtkWidget *warn = gtk_message_dialog_new (
        NULL, 0, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
        "An error occured during the system setup.\n\n"
        "No change has been done.");
    gtk_window_set_title (GTK_WINDOW (warn), "Setup error");
    gtk_dialog_run (GTK_DIALOG (warn));
}

const gchar *
get_selected_image (GtkComboBox *combo)
{
    GtkTreeModel *model;
    GtkTreeIter iter;
    gchar *image_id = NULL;
    const gchar *ret;

    model = gtk_combo_box_get_model (combo);
    gtk_combo_box_get_active_iter (combo, &iter);
    gtk_tree_model_get (model, &iter, 0, &image_id, -1);

    ret = g_strdup (image_id);
    g_free (image_id);

    return ret;
}

